

; ==============================================================================
; Blitz3D GeoWall Game Engine Template
; (C) 2004 Jason Leigh, Electronic Visualization Laboratory
;
; Use this to get started on your Blitz3D GeoWall Game
; This code shows:
;	1. how to set up GeoWall stereo
;	2. how to create a game loop that will keep constant update rate on CPUs with different speeds
;	3. loads a simple animated test object
;
; Keys:
;	1 = mono
;	2 = stereo
;	3 = reverse stereo
;	+ = increase stereo separation
;	- = decrease sterep separation
;
; Created: 7/15/2004
; Version: 8/17/2004
; For comments email: spiff@evl.uic.edu
; -----------------------------------------------------------------------------




; Include geowall stereo library
Include "BlitzGeoWallStereoLib.bb"


; ==============================================================================
; Init graphics and stereo cameras
Global g_screenXSize = 1024
Global g_screenYSize = 768

Print "Choose Display Mode:"
Print "1 - Standard GeoWall Stereo (2048x768)"
Print "2 - Desktop (1280x1024)"
Print "3 - Desktop (1600x1200)"
Print "4 - Desktop (1024x768)"

choice=WaitKey()

Select choice

Case 49
	g_screenXSize=2048
	g_screenYSize=768

Case 50
	g_screenXSize=1280
	g_screenYSize=1024

Case 51
	g_screenXSize=1600
	g_screenYSize=1200

Default
	g_screenXSize=1024
	g_screenYSize=768
End Select


Graphics3D g_screenXSize, g_screenYSize

; Eye separation (-ve means cross-eyed stereo) 
Global g_eyeSeparation#=0.2
Global g_reverseStereo#=1
SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)

; Stereo viewing flag: 0=stereo 1=mono
Global g_viewMode=0
SetBuffer BackBuffer() 

; This is the main camera that you control in the game.
Global g_mainCamera = SL_CreateStereoCameras()
SL_SetUpCameras()

camera=SL_GetRightCamera()
CameraRange camera,0.1,500000
camera=SL_GetLeftCamera()
CameraRange camera,0.1,500000

PositionEntity g_mainCamera,0,1,-10 

light=CreateLight() 
RotateEntity light,90,0,0 

plane=CreatePlane() 
ground_tex=LoadTexture("media/floor.jpg") 
EntityTexture plane,ground_tex 

cube=CreateCube() 
cube_tex=LoadTexture("media/cube.jpg") 
EntityTexture cube,cube_tex 
PositionEntity cube,0,1,0 

microphone=CreateListener(camera) ; Create listener, make it child of camera 
sound=Load3DSound("media/sound.wav") ; Load 3D sound 

LoopSound sound
EmitSound (sound, cube)

;FreeEntity microphone
;microphone = CreateListener(camera)

; ==============================================================================
; Set the game update/calculation rate to 30 frames per second.
; This is NOT the same as FRAME RATE which marks how rapidly the graphics image is updating.
; In this code, the game update/calculation rate is locked down so that the game behaves
; the same for different speed of computers, but the graphics always refreshes as fast as possible.
Const c_GameRate=30
period=1000/c_GameRate
;theTime=MilliSecs()-period
theTime=MilliSecs()

; ==============================================================================

; ==============================================================================
;
;                      *******  MAIN GAME LOOP  *******
;
; ==============================================================================
; MAIN GAME LOOP that maintains stable frame rate for all parts of the game
;
While Not KeyHit( 1 ) 

	; Compute how much time has elapsed thru the entire main loop
	Repeat
		elapsedTime=MilliSecs()-theTime
	Until elapsedTime<>0
	
	;how many 'frames' have elapsed	
	elapsedFrames=elapsedTime/period
	
	; If more than 1 frame has elapsed then we have to compute
	; where what the state of the world would have been.
	; This is crucial for slower computers.
	; So this loop goes thru each frame of missed time and computes
	; what the new state of the world should be by simplying
	; calling the users UpdateGameState elapsedFrames times.
	For k=1 To elapsedFrames
	
		; When elapsedFrames is reached, save the state of the world so that tweening
		; can be used to get an accurate interpolation of what the state of the world
		; should be.
		If k=elapsedFrames Then CaptureWorld
		theTime=theTime+period
		
		; Update the game's state (e.g. position of each entity)		
		GameProcess()

		; Update all animation properties- e.g. compute collision checks
		UpdateWorld
	Next

	;Compute the tweening value (a number from 0 to 1)
	; 0 means the state of the world when CaptureWorld was called
	; 1 means the current interpolated state of the world
	; e.g. if elapsedTime is 1 second and desired period is 3 second
	; then 1mod3=1 hence tween=1/period = 1/3 
	; so we interpolate 1/3 distance from the captured point and the present time.
	;
	; if elapsedTime = 5 seconds and desired period is 3 seconds;
	; 1 frame of time would have passed so it would have gone thru
	; the for loop once computing the new frame state
	; Since 5mod3=2 hence tween=2/3 so we tween 2/3 the way between the captured state
	; and the current state.
	tween#=Float(elapsedTime Mod period)/Float(period)


	; Switch to mono mode by pressing 1
	If KeyHit(2)>0 Then
	g_viewMode=1
		SL_SwitchToMono()
	EndIf
	
	; Switch to stereo mode by pressing 2
	If KeyHit(3)>0 Then
	g_viewMode=0
		g_reverseStereo#=1
		SL_SwitchToStereo()
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf

	; Switch to cross eye stereo mode by pressing 3
	If KeyHit(4)>0 Then
		g_viewMode=0
		g_reverseStereo#=-1
		SL_SwitchToStereo()
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	; Change eye separation by pressing + and -
	If KeyHit(13)>0 Then
		g_eyeSeparation#=g_eyeSeparation#+0.1
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	If KeyHit(12) > 0 Then
		g_eyeSeparation#=g_eyeSeparation#-0.1
		If g_eyeSeparation# < 0 Then g_eyeSeparation#=0
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	

	; Adjust stereo cameras and render the world
	SL_Update()
	RenderWorld tween#
	

	; Drawing occurs to the backbuffer, so you need to flip the backbuffer
	; to the front to see it. (see explanation in class lecture.)
	Flip


Wend 


; ***********************************************************************
; DO YOUR OWN STUFF HERE!!!!!!
Function GameProcess()

If KeyDown(205)=True Then TurnEntity g_mainCamera,0,-1,0 
If KeyDown(203)=True Then TurnEntity g_mainCamera,0,1,0 
If KeyDown(208)=True Then MoveEntity g_mainCamera,0,0,-0.05 
If KeyDown(200)=True Then MoveEntity g_mainCamera,0,0,0.05 

	
End Function


End